__title__ = 'FinDataBroker'
__description__ = 'A databroker for financial data'
__url__ = 'https://github.com/tomerten/findatabroker'
__version__ = '1.0.0alpha4'
__author__ = 'Tom Mertens'
__license__ = 'GNU General Public License v3.0'


from typing import List, Dict

import dataset
from tqdm import tqdm

from .Template import DataBroker


class DataBrokerSqlite(DataBroker):
    """
    Class to handle saving FinanceJSON data into an
    sqlite database.
    """

    def __init__(self, sqliteuri):
        super(DataBrokerSqlite, self).__init__()
        self.db = dataset.connect(sqliteuri)

    def _create_index(self, table: dataset.table, indexList: List[str]):
        table.create_index(indexList, name='main')

    def save(self, data: List[Dict], collection: 'str', indexList: List[str]):
        table = self.db.create_table(collection)
        if not isinstance(data, list):
            data = [data]

        for row in tqdm(data):
            try:
                table.insert_ignore(row, indexList, ensure=True)
            except:
                print(",".join(indexList))
                print(row)
                break

        self._create_index(table, indexList)

    def load(self, collection: str, searchdict: Dict, *args, **kwargs):
        try:
            table = list(self.db.load_table(collection).find(**searchdict))
            for t in table:
                t.pop('id')
            return table
        except:
            print(f'Table {collection} not found')



from json import dump

import dataset
from click.testing import CliRunner
from financejson.validate import validate_file
from pymongo import MongoClient
from FinDataBroker.CLI import write2mongo, write2sqlite


def test_write2mongo_invalid_CLI():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(write2mongo, ['test.json', 'boe'])
        assert result.exit_code == 2


def test_write2mongo_no_valid_file_does_not_exist():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(write2mongo, ['test.json', 'boe', 'boe'])
        assert result.exit_code == 2


def test_write2mongo_invalid_file():
    runner = CliRunner()
    with runner.isolated_filesystem():
        data = {
            "yh_symbol": [{"symbol": "XYZ"}]
        }
        with open('test.json', 'w') as f:
            dump(data, f)
        result = runner.invoke(write2mongo, ['test.json', 'boe', 'boe'])
        assert result.exit_code == 1

# def test_write2mongo_valid_file():
#     runner = CliRunner()
#
#     with runner.isolated_filesystem():
#         data = {
#             "yh_symbol": [{"symbol": "AAPL"}],
#             "ms_symbol": [{"symbol": "US_AAPL"}],
#             "yh_assetProfile": [
#                 {
#                     "index_symbol": "XYZ",
#                     "date": "2019-01-01",
#                     "address1": "foo",
#                     "auditRisk": 1,
#                     "boardRisk": 2,
#                     "city": "bar",
#                     "country": "a"
#                 }],
#         }
#
#         with open('test.json', 'w') as f:
#             dump(data, f)
#         validate_file('test.json')
#         result = runner.invoke(write2mongo, ['test.json', 'localhost:27017', 'boe'])
#         assert result.exit_code == 0
#
#         client = MongoClient('localhost:27017')
#
#         stored = list(client['boe']['yh_assetProfile'].find({}))
#         for elem in stored:
#             elem.pop('_id')
#
#         assert data['yh_assetProfile'] == stored
#
#         client.drop_database('boe')
#         client.close()


def test_write2sqlite_invalid_CLI():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(write2mongo, ['test.json'])
        assert result.exit_code == 2


def test_write2sqlite_no_valid_file_does_not_exist():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(write2mongo, ['test.json', 'boe'])
        assert result.exit_code == 2


def test_write2sqlite_invalid_file():
    runner = CliRunner()
    with runner.isolated_filesystem():
        data = {
            "yh_symbol": [{"symbol": "XYZ"}]
        }
        with open('test.json', 'w') as f:
            dump(data, f)
        result = runner.invoke(write2mongo, ['test.json', 'boe'])
        assert result.exit_code == 2


def test_write2sqlite_valid_file():
    runner = CliRunner()

    with runner.isolated_filesystem():
        data = {
            "yh_symbol": [{"symbol": "AAPL"}],
            "ms_symbol": [{"symbol": "US_AAPL"}],
            "yh_assetProfile": [
                {
                    "index_symbol": "XYZ",
                    "date": "2019-01-01",
                    "address1": "foo",
                    "auditRisk": 1,
                    "boardRisk": 2,
                    "city": "bar",
                    "country": "a"
                }],
        }

        with open('test.json', 'w') as f:
            dump(data, f)
        validate_file('test.json')

        result = runner.invoke(write2sqlite, ['test.json', 'sqlite:///test.db'])
        assert result.exit_code == 0

        db = dataset.connect('sqlite:///test.db')
        stored = list(db['yh_assetProfile'])

        for elem in stored:
            elem.pop('id')
        assert data['yh_assetProfile'] == stored



import pytest, os
from FinDataBroker.DataBrokerSqlite import DataBrokerSqlite


# @pytest.fixture(scope='function')
# def reset_sqlite_db(request):
#     path = request.param  # Path to database file
#     with open(path, 'w'): pass
#     yield None
#     os.remove(path)

# @pytest.fixture(autouse=True)
# def reset(reset_sqlite_db):
#     reset_sqlite_db.reset()

@pytest.fixture(scope='function')
def db(tmpdir):
    file = os.path.join("sqlite:///test.db")
    print(file)
    yield file
    os.remove('test.db')


# @pytest.mark.parametrize('reset_sqlite_db', ['/tmp/test_db.sql'], indirect=True)
def test_create_database_no_tables___ok(db):
    dbsql = DataBrokerSqlite(db)

    assert dbsql.db.tables == []


# @pytest.mark.parametrize('reset_sqlite_db', ['/tmp/test_db.sql'], indirect=True)
def test_create_database_create_table___ok(db):
    dbsql = DataBrokerSqlite(db)
    dbsql.save({}, 'test', ['year'])

    assert dbsql.db.tables == ['test']
    assert len(dbsql.db['test']) == 1


def test_create_database_create_table_with_data__ok(db):
    dbsql = DataBrokerSqlite(db)
    dbsql.save({"boe": "hallo"}, 'test', ['boe'])

    assert dbsql.db.tables == ['test']
    assert dbsql.db["test"].columns == ['id', "boe"]
    assert len(dbsql.db["test"]) == 1

    dbsql.save({"year": 2019}, 'test', ['boe', 'year'])

    assert len(dbsql.db["test"]) == 2
    assert dbsql.db["test"].columns == ['id', "boe", 'year']

    boe = dbsql.db['test'].find_one(boe="hallo")

    assert boe["boe"] == "hallo"


def test_load_ok(db):
    dbsql = DataBrokerSqlite(db)
    dbsql.save({"boe": "hallo"}, 'test', ['boe'])

    assert dbsql.db.tables == ['test']
    assert dbsql.db["test"].columns == ['id', "boe"]
    assert len(dbsql.db["test"]) == 1

    dbsql.save({"year": 2019}, 'test', ['boe', 'year'])
    retrieved = dbsql.load('test', {'boe': 'hallo'})
    assert len(retrieved) == 1
    assert retrieved[0]['boe'] == 'hallo'

    dbsql.save({"year": 2019}, 'test', ['boe', 'year'])
    retrieved = dbsql.load('test', {'boe': 'hallo'})
    assert len(retrieved) == 1
    assert retrieved[0]['boe'] == 'hallo'
    assert len(dbsql.db["test"]) == 2



from pymongo import ASCENDING

# index_symbol is yahoo ticker symbol
indexMap = {
    "yh_symbol": [('symbol',  ASCENDING)],
    "ms_symbol": [('symbol',  ASCENDING)],
    "yh_currency": [('index_symbol',  ASCENDING)],
    "ms_currency": [('index_symbol',  ASCENDING)],
    "yh_ohlcv_3mo": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_1mo": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_1wk": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_5d": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_1d": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_1h": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_90m": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_60m": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_30m": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_15m": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_5m": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_2m": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_ohlcv_1m": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_assetProfile": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_assetProfile_companyOfficers": [
        ('index_symbol',  ASCENDING),
        ('name',  ASCENDING)
    ],
    "yh_balanceSheetStatementsYearly": [
        ('index_symbol',  ASCENDING),
        ('endDate',  ASCENDING)
    ],
    "yh_balanceSheetStatementsQuarterly": [
        ('index_symbol',  ASCENDING),
        ('endDate',  ASCENDING)
    ],
    "yh_calendarEvents": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_calendarEvents_earnings": [
        ('index_symbol',  ASCENDING),
        ('earningsDate',  ASCENDING)
    ],
    "yh_cashflowStatementsYearly": [
        ('index_symbol',  ASCENDING),
        ('endDate',  ASCENDING)
    ],
    "yh_cashflowStatementsQuarterly": [
        ('index_symbol',  ASCENDING),
        ('endDate',  ASCENDING)
    ],
    "yh_keyStatistics": [
        ('index_symbol',  ASCENDING),
        ("lastFiscalYearEnd",  ASCENDING)
    ],
    "yh_earnings_earningsChart_quarterly": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_earnings_earningsChart": [
        ('index_symbol',  ASCENDING),
        ('earningsDate',  ASCENDING)
    ],
    "yh_earnings_financialsChart_yearly": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_earnings_financialsChart_quarterly": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_earnings": [
        ('index_symbol',  ASCENDING)
    ],
    "yh_earningsHistory": [
        ('index_symbol',  ASCENDING),
        ('quarter',  ASCENDING)
    ],
    "yh_earningsTrend": [
        ('index_symbol',  ASCENDING),
        ('endDate',  ASCENDING)
    ],
    "yh_esgScores_peerEsgScorePerformance": [
        ('index_symbol',  ASCENDING),
        ('ratingYear',  ASCENDING),
        ('ratingMonth',  ASCENDING)
    ],
    "yh_esgScores_peerGovernancePerformance": [
        ('index_symbol',  ASCENDING),
        ('ratingYear',  ASCENDING),
        ('ratingMonth',  ASCENDING)
    ],
    "yh_esgScores_peerSocialPerformance": [
        ('index_symbol',  ASCENDING),
        ('ratingYear',  ASCENDING),
        ('ratingMonth',  ASCENDING)
    ],
    "yh_esgScores_peerEnvironmentPerformance": [
        ('index_symbol',  ASCENDING),
        ('ratingYear',  ASCENDING),
        ('ratingMonth',  ASCENDING)
    ],
    "yh_esgScores_peerHighestControversyPerformance": [
        ('index_symbol',  ASCENDING),
        ('ratingYear',  ASCENDING),
        ('ratingMonth',  ASCENDING)
    ],
    "yh_esgScores_relatedControversy": [
        ('index_symbol',  ASCENDING),
        ('ratingYear',  ASCENDING),
        ('ratingMonth',  ASCENDING)
    ],
    "yh_esgScores": [
        ('index_symbol',  ASCENDING),
        ('ratingYear',  ASCENDING),
        ('ratingMonth',  ASCENDING)
    ],
    "yh_financial_data": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_fundOwnership_ownershipList": [
        ('index_symbol',  ASCENDING),
        ('reportDate',  ASCENDING)
    ],
    "yh_incomeStatementHistory": [
        ('index_symbol',  ASCENDING),
        ('endDate',  ASCENDING)
    ],
    "yh_incomeStatementHistoryQuarterly": [
        ('index_symbol',  ASCENDING),
        ('endDate',  ASCENDING)
    ],
    "yh_indexTrend_estimates": [
        ('index_symbol',  ASCENDING),
        ('date', ASCENDING),
        ('period',  ASCENDING)
    ],
    # 02-01-2020 : added combination of period and date as index
    # otherwise next quarter and 0y not unique.
    "yh_indexTrend": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING),
    ],
    # 02-01-2020 : added combination of period and date as index
    # otherwise next quarter and 0y not unique.
    "yh_industryTrend_estimates": [
        ('index_symbol',  ASCENDING),
        ('date', ASCENDING),
        ('period',  ASCENDING)
    ],
    "yh_industryTrend": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_insiderHolders": [
        ('index_symbol',  ASCENDING),
        ('latestTransDate',  ASCENDING),
        ('name',  ASCENDING),
        ('transactionDescription',  ASCENDING)
    ],
    "yh_insiderTransactions": [
        ('index_symbol',  ASCENDING),
        ('filerName',  ASCENDING),
        ('startDate',  ASCENDING)
    ],
    "yh_institutionOwnership": [
        ('index_symbol',  ASCENDING),
        ('organization',  ASCENDING),
        ('reportDate',  ASCENDING)
    ],
    "yh_majorDirectHolders": [
        ('index_symbol',  ASCENDING),
        ('name',  ASCENDING),
        ('reportDate',  ASCENDING)
    ],
    "yh_majorHoldersBreakdown": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_netSharePurchaseActivity": [
        ('index_symbol',  ASCENDING),
        ('period',  ASCENDING)
    ],
    "yh_price": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING)
    ],
    "yh_quoteType": [
        ('index_symbol',  ASCENDING)
    ],
    "yh_recommendationTrend": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING),
        ('period', ASCENDING)
    ],
    "yh_secFilings": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING),
        ('type',  ASCENDING)
    ],
    "yh_sectorTrend_estimates": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING),
        ('period',  ASCENDING)
    ],
    "yh_sectorTrend": [
        ('index_symbol',  ASCENDING),
        ('date',  ASCENDING),
    ],
    "yh_summaryDetail": [
        ('index_symbol',  ASCENDING),
        ('startDate',  ASCENDING)
    ],
    "yh_upgradeDowngradeHistory": [
        ('index_symbol',  ASCENDING),
        ('firm',  ASCENDING),
        ('epochGradeDate',  ASCENDING),
        ('action',  ASCENDING)
    ],
    "ms_balancesheet": [
        ('index_symbol',  ASCENDING),
        ('year',  ASCENDING)
    ],
    "ms_profitabilityRatios": [
        ('index_symbol',  ASCENDING),
        ('year',  ASCENDING)
    ],
    "ms_cashflowsheet": [
        ('index_symbol',  ASCENDING),
        ('year',  ASCENDING)
    ],
    "ms_liquidity": [
        ('index_symbol',  ASCENDING),
        ('year',  ASCENDING)
    ],
    "ms_financials": [
        ('index_symbol',  ASCENDING),
        ('year',  ASCENDING)
    ],
}



import click
from json import load
from pymongo import MongoClient

from financejson.validate import validate_file
from FinDataBroker.DataBrokerMongoDb import DataBrokerMongoDb
from FinDataBroker.DataBrokerSqlite import DataBrokerSqlite
from FinDataBroker.IndexingMap import indexMap


@click.group()
@click.version_option()
def main():
    pass


@main.command()
@click.argument('file', type=click.Path(exists=True))
@click.argument('mongouri')
@click.argument('dbname')
def write2mongo(**kwargs):
    print(kwargs)
    # check if json file is valid
    validate_file(kwargs['file'])

    # load the data into a dictionary
    with open(kwargs['file'], 'r') as f:
        data = load(f)

    # create a mongodb databroker instance
    client = MongoClient(kwargs['mongouri'])
    dbm = DataBrokerMongoDb(client)

    # write data to mongodb
    # get rid of auxiliary data
    data.pop('yh_symbol')
    data.pop('ms_symbol')
    # loop through the data fielsd
    for k, v in data.items():
        dbm.save(v, db=kwargs['dbname'], col=k, indexTupleList=indexMap[k], unique=True)

    client.close()


@main.command()
@click.argument('file', type=click.Path(exists=True))
@click.argument('sqliteuri')
def write2sqlite(**kwargs):
    # check if json file is valid
    validate_file(kwargs['file'])

    # load the data into a dictionary
    with open(kwargs['file'], 'r') as f:
        data = load(f)

    # create a sqlite databroker instance
    dbs = DataBrokerSqlite(kwargs['sqliteuri'])

    # write data to sqlite
    # get rid of auxiliary data
    data.pop('yh_symbol')
    data.pop('ms_symbol')

    # adapt index map to sql
    indexMapSQL = dict()
    for k, v in indexMap.items():
        _list = []
        for element in v:
            _list.append(element[0])
        indexMapSQL[k] = _list

    # loop through the data fielsd
    for k, v in data.items():
        dbs.save(v, collection=k, indexList=indexMapSQL[k])

    # dbs.db.close()



import pytest
from pytest import raises
import mongomock
from pymongo import ASCENDING
from FinDataBroker.DataBrokerMongoDb import DataBrokerMongoDb

client = mongomock.MongoClient()
broker = DataBrokerMongoDb(client)


def test___databroker_creation___ok():
    assert isinstance(broker, DataBrokerMongoDb)


# not implemented in mongomock
# def test___get_stat_non_existing_db___nog():
#     with raises(AttributeError):
#         broker.get_stats('test')

def test__save___ok():
    db = 'testdb'
    col = 'testcol'
    objs = [
        {
            'name': 'boe',
            'year': 2019
        }
    ]
    index = [('name', ASCENDING), ('year', ASCENDING)]

    broker.save(objs, db, col, index, unique=True)
    assert broker.get_number_of_documents(db, col) == 1


def test__load___ok():
    db = 'testdb'
    col = 'testcol'
    objs = [
        {
            'name': 'boe',
            'year': 2019
        }
    ]
    index = [('name', ASCENDING), ('year', ASCENDING)]

    broker.save(objs, db, col, index, unique=True)
    assert broker.get_number_of_documents(db, col) == 1

    data = broker.load(db, col, {'year': 2019})
    assert objs == data



from setuptools import setup, find_packages

with open('FinDataBroker/__about__.py') as file:
    about = {}
    exec(file.read(), about)

with open('README.md') as file:
    readme = file.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__description__'],
    long_description=readme,
    long_description_content_type='text/markdown',
    url=about['__url__'],
    author=about['__author__'],
    license=about['__license__'],
    packages=find_packages(),
    install_requires=requirements,
    test_requires=['pytest'],
    python_requires='>=3.6',
    include_package_data=True,
    entry_points={'console_scripts': ["FinDataBroker = FinDataBroker.cli:main"]},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Office/Business :: Financial',
        'Intended Audience :: Financial and Insurance Industry'
    ],
)



from .Template import DataBroker
from pprint import pprint
from typing import List, Tuple, Dict
from tqdm import tqdm
from termcolor import colored


class DataBrokerMongoDb(DataBroker):
    """
    DataBroker that interacts with mongodb.
    """

    def __init__(self, client):
        super(DataBrokerMongoDb, self).__init__()
        self.client = client

    def get_stats(self, db):
        """
        Prints the collections in the given database and
        the dbstats.

        :param db: database name
        :type db: str
                database name for which one
                would like to get the stats
        """
        dbm = self.client[db]

        print(dbm.collection_names())
        status = dbm.command('dbstats')

        pprint(status)

    def get_number_of_documents(self, db: str, col: str) -> int:
        """
        Get the total number of documents
        stored in the collection col
        from the database db
        :param db: database name
        :type db: str
        :param col: collection name
        :type col: str

        :return: number of documents stored in col
        """
        colm = self.client[db][col]
        n = colm.count_documents({})

        return n

    def _create_index(self, db: str, col: str, indexTupleList: List[Tuple], unique=True):
        """
        Private method to create index if
        it does not already exists.

        :param db: database name
        :type db: str
        :param col: collection name
        :type col: str
        :param indexTupleList: list of tuples to create index
        :type indexTupleList: list of tuples
              the tuples are (field, ASCENDING/DESCENDING)
        :param unique:
        :type unique: bool
                Default : True (allows double check to prevent duplicates)

        """
        colm = self.client[db][col]
        colm.create_index(indexTupleList, unique=unique)

    def save(self, data: List[Dict], db: str, col: str, indexTupleList: List[Tuple], unique=True):
        """
        Public method to save the data to
        a collection.

        User should take care that data is given
        in the correct format (list of dicts, bjson)

        :param data: data to save
        :type data: list of dicts (bjson)
        :param db: database name
        :type db: str
        :param col: collection name
        :type col: str
        :param indexTupleList:tuples define the index
        :type indexTupleListlist of tuples
        :param unique: index keys unique ? (default: True)
        :type unique:bool
        """
        self._create_index(db, col, indexTupleList, unique=unique)
        colm = self.client[db][col]
        # ref : https://docs.mongodb.com/php-library/master/reference/method/MongoDBCollection-insertMany/
        # ordered False will continue writing if one fails
        # does not seem to work properly - TODO: checkout if this can be improved
        temp = None
        if not isinstance(data, list):
            data = [data]

        for row in tqdm(data):
            try:
                if colm.count_documents({k[0]: row[k[0]] for k in indexTupleList}, limit=1) == 0:
                    colm.insert_one(row)
                else:
                    if temp == row[indexTupleList[0][0]]:
                        continue
                    else:
                        temp = row[indexTupleList[0][0]]

                        _list = [str(row[k[0]]) for k in indexTupleList]
                        tup = ','.join(_list)
                        print(colored(f"{tup} already exists", 'red'))
            except:
                print([k[0] for k in indexTupleList])
                print(row)
                break
            # colm.insert_many(data, ordered=False)

    def load(self, db: str, col: str, searchdict: Dict):
        """
        Publid method to load data from the database.

        :param db: database name
        :type db: str
        :param col: collection name
            str
        :param searchdict: mongo valid search dict
            dict

        :return: requested data as list of dicts
        """
        colm = self.client[db][col]
        return list(colm.find(searchdict, {'_id': False}))



from abc import ABC, abstractmethod


class DataBroker(ABC):
    """Abstract class for storing data."""

    def __init__(self):
        super(DataBroker, self).__init__()

    @abstractmethod
    def load(self, *args, **kwargs):
        """
        Load the saved data.
        """
        pass

    @abstractmethod
    def save(self, *args, **kwargs):
        """
        Save the data to
        given format by the
        chosen DataBroker.
        """
        pass
